@@ -1,5 +1,5 @@
 package Devel::IPerl::Kernel::Backend::Reply;
-$Devel::IPerl::Kernel::Backend::Reply::VERSION = '0.003';
+$Devel::IPerl::Kernel::Backend::Reply::VERSION = '0.004';
 use strict;
 use warnings;
 
@@ -15,15 +15,28 @@ sub _build_repl {
 	require Reply;
 	$log->trace('Creating REPL: Reply');
 
+	my @autocomplete_plugins = qw( Autocomplete::Functions Autocomplete::Globals
+		Autocomplete::Keywords Autocomplete::Lexicals Autocomplete::Methods
+		Autocomplete::Packages);
 	my $repl = Reply->new(
 		# needs these at a minimum
-		plugins => [ qw[IPerl Packages LexicalPersistence Hints] ]
-		
+		plugins => [ qw[IPerl Packages LexicalPersistence Hints],
+			@autocomplete_plugins ]
+
+
 	);
 
 	$repl;
 }
 
+sub completion {
+	my ($self, $line, $cursor_location) = @_;
+
+	my $repl = $self->repl;
+	my $line_up_to_cursor = substr $line, 0, $cursor_location;
+	my @matches = $repl->_concatenate_plugin('tab_handler', $line_up_to_cursor);
+}
+
 sub run_line {
 	my ($self, $cmd) = @_;
 	my $repl = $self->repl;
@@ -97,7 +110,7 @@ Devel::IPerl::Kernel::Backend::Reply
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 AUTHOR
 
